`include "defines.vh"
module gpio_sram #(
        parameter DATA_WIDTH    = 8,
        parameter ADDR_WIDTH    = 6   // 64 I/O registers
    )(
        input  wire                  clk,
        input  wire                  oe,
        input  wire                  cs,
        input  wire                  we,
        input  wire [ADDR_WIDTH-1:0] address,
        inout  wire [DATA_WIDTH-1:0] data,
        /* TODO 1: porta si portb trebuie sa fie bidirectionale.
  		  Din aceasta cauza nu mai pot fi reg.
  		  porta si portb reprezinta fire ce fac legatura dintre
  		  memoria IO si exteriorul chipului (pinii fizici).
  		  A NU se confunda porta (care reprezinta fire catre pini),
  		  cu MEMORY[`PORTA], care este un REGISTRU din memoria IO.
  		  */
        output  reg [DATA_WIDTH-1:0] porta,
        output  reg [DATA_WIDTH-1:0] portb
        //output wire [DATA_WIDTH-1:0] sreg
    );

    //assign sreg = memory[`SREG];
    reg [DATA_WIDTH-1:0] memory[0:(1<<ADDR_WIDTH)-1];
    reg [ADDR_WIDTH-1:0] buffer;
    /* TODO 1: declarati cate un buffer pentru porta si portb.
		Necesitatea lor este expusa mai jos.
	 */

    always @(negedge clk)
    begin
        if (cs) begin
            if (we) begin
                memory[address] <= data;
            end else begin
                buffer <= address;
            end
        end
    end

    assign data = (cs && oe && !we) ? memory[buffer] : {DATA_WIDTH{1'bz}};

    reg [DATA_WIDTH:0] i;
    always @(posedge clk) begin
        for (i = 0; i < DATA_WIDTH; i = i + 1) begin
            porta[i] <= memory[`DDRA][i] ? memory[`PORTA][i] : 1'bz;
            portb[i] <= memory[`DDRB][i] ? memory[`PORTB][i] : 1'bz;
            /* TODO 1: Atentie! Elementele urmatoare sunt expuse pentru Portu A,
    				dar sunt valabile si trebuie aplicate, de asemenea, pt Portul B.

    				Amintim ca:
    					DDRA - seteaza directia pinilor, deci:
    						DDRA[5] == 0 inseamna ca de pe pinul 5 al portului A se citeste.
    						DDRA[3] == 1 inseamna ca pe pinul 3 al portului A se scrie.
    						PORTA[3] acest bit seteaza valoarea pinului 3, daca DDRA[3] == 1 (altfel nu are efect).
    						PINA[6] ne spune valoarea de pe pinul 6 (PA6).
    				*/

    				/* TODO 1:
    					Pentru fiecar bit, daca DDRA este setat pt citire, stocam valoarea din porta in memorie.
    					(In ce registru o stocam? De unde CITIM valoarea unui pin?).
    					Daca DDRA este setata pt scriere, stocam in porta_buffer valoare ce vrem sa fie pusa pe pin.
    					(Cu ce registru setam VALOAREA ce vrem sa fie pusa pe pin? )

    					Asemeni si pt Portul B.
    				*/

        end


    end

    /* TODO 1: Asignati valori de iesire pentru fiecare pin (individual) din  porta si portb.
		Daca bitul corespunzator din DDRA este setat, vom asigna lui porta valoarea bit-ului din
		bufferul lui porta (declarate la linia 31), altfel pinul va avea 'impedanta marita' (1'bz).
	 */


endmodule
